#include "compute_pipeline.h"

#include "application/application.h"
#include <vulkan/vulkan.hpp>

#include "renderer_vk.h"

void compute_pipeline::add_binding(uint32_t binding, vk::DescriptorType descriptor_type, uint32_t descriptor_count, const vk::Sampler* immutable_samplers) {
    const vk::ShaderStageFlags flag = vk::ShaderStageFlagBits::eCompute;

    vk::DescriptorSetLayoutBinding descriptor_set_layout_binding;
    descriptor_set_layout_binding.setBinding(binding);
    descriptor_set_layout_binding.setDescriptorType(descriptor_type);
    descriptor_set_layout_binding.setDescriptorCount(descriptor_count);
    descriptor_set_layout_binding.setStageFlags(flag);
    descriptor_set_layout_binding.setPImmutableSamplers(immutable_samplers);
    descriptor_set_layout_bindings_.push_back(descriptor_set_layout_binding);
}

void compute_pipeline::create() {
    const auto renderer_vk = application::get()->get_renderer();
    auto device = renderer_vk->device;

    vk::PipelineCacheCreateInfo pipeline_cache_create_info;
    pipeline_cache_create_info.setInitialDataSize(0);
    pipeline_cache_ = device.createPipelineCache(pipeline_cache_create_info);

    create_pipeline_layout();

    vk::ComputePipelineCreateInfo compute_pipeline_create_info;
    compute_pipeline_create_info.setLayout(pipeline_layout_);
    const auto pipeline_result = device.createComputePipeline(pipeline_cache_, compute_pipeline_create_info);
    check_vk_result(pipeline_result.result);
    pipeline_ = pipeline_result.value;
}

void compute_pipeline::destroy() {
    const auto renderer_vk = application::get()->get_renderer();
    auto device = renderer_vk->device;

    device.destroyPipeline(pipeline_);
    device.destroyPipelineCache(pipeline_cache_);
    device.destroyPipelineLayout(pipeline_layout_);
}

void compute_pipeline::bind(vk::CommandBuffer command_buffer) const {
    command_buffer.bindPipeline(vk::PipelineBindPoint::eCompute, pipeline_);
}
